@@ -28,12 +28,11 @@
 #include "gdbusintrospection.h"
 
 #include "glibintl.h"
-#include "gioalias.h"
 
 /**
  * SECTION:gdbusintrospection
- * @title: Introspection XML
- * @short_description: Parse and Generate Introspection XML
+ * @title: D-Bus Introspection Data
+ * @short_description: Node and interface description data structures
  * @include: gio/gio.h
  *
  * Various data structures and convenience routines to parse and
@@ -1826,6 +1825,7 @@ g_dbus_node_info_new_for_xml (const gchar  *xml_data,
   GMarkupParser *parser;
   guint num_nodes;
   ParseData *data;
+  GDBusNodeInfo **ughret;
 
   ret = NULL;
   parser = NULL;
@@ -1848,7 +1848,6 @@ g_dbus_node_info_new_for_xml (const gchar  *xml_data,
                                      error))
     goto out;
 
-  GDBusNodeInfo **ughret;
   ughret = parse_data_steal_nodes (data, &num_nodes);
 
   if (num_nodes != 1)
@@ -2074,6 +2073,3 @@ g_dbus_node_info_lookup_interface (const GDBusNodeInfo *info,
  out:
   return result;
 }
-
-#define __G_DBUS_INTROSPECTION_C__
-#include "gioaliasdef.c"
