@@ -40,7 +40,6 @@
 #include "gcancellable.h"
 
 #include "glibintl.h"
-#include "gioalias.h"
 
 /**
  * SECTION:gdbusproxy
@@ -833,7 +832,7 @@ process_get_all_reply (GDBusProxy *proxy,
   gchar *key;
   GVariant *value;
 
-  if (strcmp (g_variant_get_type_string (result), "(a{sv})") != 0)
+  if (!g_variant_is_of_type (result, G_VARIANT_TYPE ("(a{sv})")))
     {
       g_warning ("Value for GetAll reply with type `%s' does not match `(a{sv})'",
                  g_variant_get_type_string (result));
@@ -843,14 +842,28 @@ process_get_all_reply (GDBusProxy *proxy,
   g_variant_get (result, "(a{sv})", &iter);
   while (g_variant_iter_next (iter, "{sv}", &key, &value))
     {
-      /*g_print ("got %s -> %s\n", key, g_variant_print (value, FALSE));*/
-
       g_hash_table_insert (proxy->priv->properties,
                            key, /* adopts string */
                            value); /* adopts value */
     }
   g_variant_iter_free (iter);
 
+  /* Synthesize ::g-properties-changed changed */
+  if (g_hash_table_size (proxy->priv->properties) > 0)
+    {
+      GVariant *changed_properties;
+      const gchar *invalidated_properties[1] = {NULL};
+
+      g_variant_get (result,
+                     "(@a{sv})",
+                     &changed_properties);
+      g_signal_emit (proxy, signals[PROPERTIES_CHANGED_SIGNAL],
+                     0,
+                     changed_properties,
+                     invalidated_properties);
+      g_variant_unref (changed_properties);
+    }
+
  out:
   ;
 }
@@ -932,7 +945,6 @@ on_name_owner_changed (GDBusConnection *connection,
                        gpointer          user_data)
 {
   GDBusProxy *proxy = G_DBUS_PROXY (user_data);
-  const gchar *name;
   const gchar *old_owner;
   const gchar *new_owner;
 
@@ -945,19 +957,45 @@ on_name_owner_changed (GDBusConnection *connection,
 
   g_variant_get (parameters,
                  "(&s&s&s)",
-                 &name,
+                 NULL,
                  &old_owner,
                  &new_owner);
 
-  /* we only care about a specific name */
-  if (g_strcmp0 (name, proxy->priv->name) != 0)
-    goto out;
-
   if (strlen (new_owner) == 0)
     {
       g_free (proxy->priv->name_owner);
       proxy->priv->name_owner = NULL;
-      g_hash_table_remove_all (proxy->priv->properties);
+
+      /* Synthesize ::g-properties-changed changed */
+      if (!(proxy->priv->flags & G_DBUS_PROXY_FLAGS_DO_NOT_LOAD_PROPERTIES) &&
+          g_hash_table_size (proxy->priv->properties) > 0)
+        {
+          GVariantBuilder builder;
+          GVariant *changed_properties;
+          GPtrArray *invalidated_properties;
+          GHashTableIter iter;
+          const gchar *key;
+
+          /* Build changed_properties (always empty) and invalidated_properties ... */
+          g_variant_builder_init (&builder, G_VARIANT_TYPE ("a{sv}"));
+          changed_properties = g_variant_builder_end (&builder);
+          invalidated_properties = g_ptr_array_new_with_free_func (g_free);
+          g_hash_table_iter_init (&iter, proxy->priv->properties);
+          while (g_hash_table_iter_next (&iter, (gpointer) &key, NULL))
+            g_ptr_array_add (invalidated_properties, g_strdup (key));
+          g_ptr_array_add (invalidated_properties, NULL);
+
+          /* ... throw out the properties ... */
+          g_hash_table_remove_all (proxy->priv->properties);
+
+          /* ... and finally emit the ::g-properties-changed signal */
+          g_signal_emit (proxy, signals[PROPERTIES_CHANGED_SIGNAL],
+                         0,
+                         changed_properties,
+                         (const gchar* const *) invalidated_properties->pdata);
+          g_variant_unref (changed_properties);
+          g_ptr_array_unref (invalidated_properties);
+        }
       g_object_notify (G_OBJECT (proxy), "g-name-owner");
     }
   else
@@ -2215,8 +2253,10 @@ g_dbus_proxy_call (GDBusProxy          *proxy,
 
   g_object_set_data_full (G_OBJECT (simple), "-gdbus-proxy-method-name", g_strdup (target_method_name), g_free);
 
-  /* Just warn here */
-  expected_method_info = lookup_method_info_or_warn (proxy, target_method_name);
+  /* Warn if method is unexpected (cf. :g-interface-info) */
+  expected_method_info = NULL;
+  if (!was_split)
+    expected_method_info = lookup_method_info_or_warn (proxy, target_method_name);
 
   if (expected_method_info)
     reply_type = _g_dbus_compute_complete_signature (expected_method_info->out_args);
@@ -2397,6 +2437,3 @@ g_dbus_proxy_call_sync (GDBusProxy      *proxy,
 }
 
 /* ---------------------------------------------------------------------------------------------------- */
-
-#define __G_DBUS_PROXY_C__
-#include "gioaliasdef.c"
