@@ -31,7 +31,6 @@
 #include "gsimpleasyncresult.h"
 #include "gioerror.h"
 
-#include "gioalias.h"
 
 /**
  * SECTION:ginputstream
@@ -91,13 +90,6 @@ static gboolean g_input_stream_real_close_finish (GInputStream         *stream,
 static void
 g_input_stream_finalize (GObject *object)
 {
-  GInputStream *stream;
-
-  stream = G_INPUT_STREAM (object);
-  
-  if (!stream->priv->closed)
-    g_input_stream_close (stream, NULL, NULL);
-
   G_OBJECT_CLASS (g_input_stream_parent_class)->finalize (object);
 }
 
@@ -355,8 +347,6 @@ g_input_stream_real_skip (GInputStream  *stream,
   char buffer[8192];
   GError *my_error;
 
-  class = G_INPUT_STREAM_GET_CLASS (stream);
-
   if (G_IS_SEEKABLE (stream) && g_seekable_can_seek (G_SEEKABLE (stream)))
     {
       if (g_seekable_seek (G_SEEKABLE (stream),
@@ -370,14 +360,14 @@ g_input_stream_real_skip (GInputStream  *stream,
   /* If not seekable, or seek failed, fall back to reading data: */
 
   class = G_INPUT_STREAM_GET_CLASS (stream);
-  
+
   read_bytes = 0;
   while (1)
     {
       my_error = NULL;
 
       ret = class->read_fn (stream, buffer, MIN (sizeof (buffer), count),
-			 cancellable, &my_error);
+                            cancellable, &my_error);
       if (ret == -1)
 	{
 	  if (read_bytes > 0 &&
@@ -387,16 +377,16 @@ g_input_stream_real_skip (GInputStream  *stream,
 	      g_error_free (my_error);
 	      return read_bytes;
 	    }
-	  
+
 	  g_propagate_error (error, my_error);
 	  return -1;
 	}
 
       count -= ret;
       read_bytes += ret;
-      
+
       if (ret == 0 || count == 0)
-	return read_bytes;
+        return read_bytes;
     }
 }
 
@@ -511,7 +501,7 @@ async_ready_close_callback_wrapper (GObject      *source_object,
  * You can then call g_input_stream_read_finish() to get the result of the 
  * operation.
  *
- * During an async request no other sync and async calls are allowed, and will
+ * During an async request no other sync and async calls are allowed on @stream, and will
  * result in %G_IO_ERROR_PENDING errors. 
  *
  * A value of @count larger than %G_MAXSSIZE will cause a %G_IO_ERROR_INVALID_ARGUMENT error.
@@ -632,8 +622,8 @@ g_input_stream_read_finish (GInputStream  *stream,
  * @callback: callback to call when the request is satisfied
  * @user_data: the data to pass to callback function
  *
- * Request an asynchronous skip of @count bytes from the stream into the buffer
- * starting at @buffer. When the operation is finished @callback will be called. 
+ * Request an asynchronous skip of @count bytes from the stream.
+ * When the operation is finished @callback will be called. 
  * You can then call g_input_stream_skip_finish() to get the result of the 
  * operation.
  *
@@ -1168,13 +1158,16 @@ close_async_thread (GSimpleAsyncResult *res,
      cancellation, since we want to close things anyway, although
      possibly in a quick-n-dirty way. At least we never want to leak
      open handles */
-  
+
   class = G_INPUT_STREAM_GET_CLASS (object);
-  result = class->close_fn (G_INPUT_STREAM (object), cancellable, &error);
-  if (!result)
+  if (class->close_fn)
     {
-      g_simple_async_result_set_from_error (res, error);
-      g_error_free (error);
+      result = class->close_fn (G_INPUT_STREAM (object), cancellable, &error);
+      if (!result)
+	{
+	  g_simple_async_result_set_from_error (res, error);
+	  g_error_free (error);
+	}
     }
 }
 
@@ -1210,6 +1203,3 @@ g_input_stream_real_close_finish (GInputStream  *stream,
   g_warn_if_fail (g_simple_async_result_get_source_tag (simple) == g_input_stream_real_close_async);
   return TRUE;
 }
-
-#define __G_INPUT_STREAM_C__
-#include "gioaliasdef.c"
