@@ -2,7 +2,6 @@
 /* Generated data (by glib-mkenums) */
 
 #include <gio.h>
-#include "gioalias.h"
 
 /* enumerations from "gioenums.h" */
 GType
@@ -16,6 +15,7 @@ g_app_info_create_flags_get_type (void)
         { G_APP_INFO_CREATE_NONE, "G_APP_INFO_CREATE_NONE", "none" },
         { G_APP_INFO_CREATE_NEEDS_TERMINAL, "G_APP_INFO_CREATE_NEEDS_TERMINAL", "needs-terminal" },
         { G_APP_INFO_CREATE_SUPPORTS_URIS, "G_APP_INFO_CREATE_SUPPORTS_URIS", "supports-uris" },
+        { G_APP_INFO_CREATE_SUPPORTS_STARTUP_NOTIFICATION, "G_APP_INFO_CREATE_SUPPORTS_STARTUP_NOTIFICATION", "supports-startup-notification" },
         { 0, NULL, NULL }
       };
       GType g_define_type_id =
@@ -896,6 +896,7 @@ g_dbus_connection_flags_get_type (void)
         { G_DBUS_CONNECTION_FLAGS_AUTHENTICATION_SERVER, "G_DBUS_CONNECTION_FLAGS_AUTHENTICATION_SERVER", "authentication-server" },
         { G_DBUS_CONNECTION_FLAGS_AUTHENTICATION_ALLOW_ANONYMOUS, "G_DBUS_CONNECTION_FLAGS_AUTHENTICATION_ALLOW_ANONYMOUS", "authentication-allow-anonymous" },
         { G_DBUS_CONNECTION_FLAGS_MESSAGE_BUS_CONNECTION, "G_DBUS_CONNECTION_FLAGS_MESSAGE_BUS_CONNECTION", "message-bus-connection" },
+        { G_DBUS_CONNECTION_FLAGS_DELAY_MESSAGE_PROCESSING, "G_DBUS_CONNECTION_FLAGS_DELAY_MESSAGE_PROCESSING", "delay-message-processing" },
         { 0, NULL, NULL }
       };
       GType g_define_type_id =
@@ -1104,8 +1105,6 @@ g_settings_bind_flags_get_type (void)
   return g_define_type_id__volatile;
 }
 
-#define __GIO_ENUM_TYPES_C__
-#include "gioaliasdef.c"
 
 /* Generated data ends here */
 
