@@ -25,6 +25,7 @@
 
 #include "gsettingsbackendinternal.h"
 #include "gnullsettingsbackend.h"
+#include "gsimplepermission.h"
 #include "giomodule-priv.h"
 #include "gio-marshal.h"
 
@@ -33,22 +34,16 @@
 #include <glib.h>
 #include <glibintl.h>
 
-#include "gioalias.h"
 
 G_DEFINE_ABSTRACT_TYPE (GSettingsBackend, g_settings_backend, G_TYPE_OBJECT)
 
-typedef struct _GSettingsBackendWatch GSettingsBackendWatch;
+typedef struct _GSettingsBackendClosure GSettingsBackendClosure;
+typedef struct _GSettingsBackendWatch   GSettingsBackendWatch;
 
 struct _GSettingsBackendPrivate
 {
   GSettingsBackendWatch *watches;
-  gchar *context;
-};
-
-enum
-{
-  PROP_0,
-  PROP_CONTEXT
+  GStaticMutex lock;
 };
 
 /**
@@ -86,61 +81,6 @@ enum
  * </para></note>
  **/
 
-struct _GSettingsBackendWatch
-{
-  GSettingsBackendChangedFunc              changed;
-  GSettingsBackendPathChangedFunc          path_changed;
-  GSettingsBackendKeysChangedFunc          keys_changed;
-  GSettingsBackendWritableChangedFunc      writable_changed;
-  GSettingsBackendPathWritableChangedFunc  path_writable_changed;
-  gpointer                                 user_data;
-
-  GSettingsBackendWatch                   *next;
-};
-
-void
-g_settings_backend_watch (GSettingsBackend                        *backend,
-                          GSettingsBackendChangedFunc              changed,
-                          GSettingsBackendPathChangedFunc          path_changed,
-                          GSettingsBackendKeysChangedFunc          keys_changed,
-                          GSettingsBackendWritableChangedFunc      writable_changed,
-                          GSettingsBackendPathWritableChangedFunc  path_writable_changed,
-                          gpointer                                 user_data)
-{
-  GSettingsBackendWatch *watch;
-
-  watch = g_slice_new (GSettingsBackendWatch);
-  watch->changed = changed;
-  watch->path_changed = path_changed;
-  watch->keys_changed = keys_changed;
-  watch->writable_changed = writable_changed;
-  watch->path_writable_changed = path_writable_changed;
-  watch->user_data = user_data;
-
-  watch->next = backend->priv->watches;
-  backend->priv->watches = watch;
-}
-
-void
-g_settings_backend_unwatch (GSettingsBackend *backend,
-                            gpointer          user_data)
-{
-  GSettingsBackendWatch **ptr;
-
-  for (ptr = &backend->priv->watches; *ptr; ptr = &(*ptr)->next)
-    if ((*ptr)->user_data == user_data)
-      {
-        GSettingsBackendWatch *tmp = *ptr;
-
-        *ptr = tmp->next;
-        g_slice_free (GSettingsBackendWatch, tmp);
-
-        return;
-      }
-
-  g_assert_not_reached ();
-}
-
 static gboolean
 is_key (const gchar *key)
 {
@@ -179,6 +119,276 @@ is_path (const gchar *path)
   return TRUE;
 }
 
+GMainContext *
+g_settings_backend_get_active_context (void)
+{
+  GMainContext *context;
+  GSource *source;
+
+  if ((source = g_main_current_source ()))
+    context = g_source_get_context (source);
+
+  else
+    {
+      context = g_main_context_get_thread_default ();
+
+      if (context == NULL)
+        context = g_main_context_default ();
+    }
+
+  return context;
+}
+
+struct _GSettingsBackendWatch
+{
+  GObject                                 *target;
+  GMainContext                            *context;
+  GSettingsBackendChangedFunc              changed;
+  GSettingsBackendPathChangedFunc          path_changed;
+  GSettingsBackendKeysChangedFunc          keys_changed;
+  GSettingsBackendWritableChangedFunc      writable_changed;
+  GSettingsBackendPathWritableChangedFunc  path_writable_changed;
+
+  GSettingsBackendWatch                   *next;
+};
+
+struct _GSettingsBackendClosure
+{
+  void (*function) (GSettingsBackend *backend,
+                    GObject          *target,
+                    const gchar      *name,
+                    gpointer          data1,
+                    gpointer          data2);
+
+  GSettingsBackend *backend;
+  GObject          *target;
+  gchar            *name;
+  gpointer          data1;
+  GBoxedFreeFunc    data1_free;
+  gpointer          data2;
+};
+
+static void
+g_settings_backend_watch_weak_notify (gpointer  data,
+                                      GObject  *where_the_object_was)
+{
+  GSettingsBackend *backend = data;
+  GSettingsBackendWatch **ptr;
+
+  /* search and remove */
+  g_static_mutex_lock (&backend->priv->lock);
+  for (ptr = &backend->priv->watches; *ptr; ptr = &(*ptr)->next)
+    if ((*ptr)->target == where_the_object_was)
+      {
+        GSettingsBackendWatch *tmp = *ptr;
+
+        *ptr = tmp->next;
+        g_slice_free (GSettingsBackendWatch, tmp);
+
+        g_static_mutex_unlock (&backend->priv->lock);
+        return;
+      }
+
+  /* we didn't find it.  that shouldn't happen. */
+  g_assert_not_reached ();
+}
+
+/*< private >
+ * g_settings_backend_watch:
+ * @backend: a #GSettingsBackend
+ * @target: the GObject (typically GSettings instance) to call back to
+ * @context: a #GMainContext, or %NULL
+ * ...: callbacks...
+ *
+ * Registers a new watch on a #GSettingsBackend.
+ *
+ * note: %NULL @context does not mean "default main context" but rather,
+ * "it is okay to dispatch in any context".  If the default main context
+ * is specifically desired then it must be given.
+ *
+ * note also: if you want to get meaningful values for the @origin_tag
+ * that appears as an argument to some of the callbacks, you *must* have
+ * @context as %NULL.  Otherwise, you are subject to cross-thread
+ * dispatching and whatever owned @origin_tag at the time that the event
+ * occured may no longer own it.  This is a problem if you consider that
+ * you may now be the new owner of that address and mistakenly think
+ * that the event in question originated from yourself.
+ *
+ * tl;dr: If you give a non-%NULL @context then you must ignore the
+ * value of @origin_tag given to any callbacks.
+ **/
+void
+g_settings_backend_watch (GSettingsBackend                        *backend,
+                          GObject                                 *target,
+                          GMainContext                            *context,
+                          GSettingsBackendChangedFunc              changed,
+                          GSettingsBackendPathChangedFunc          path_changed,
+                          GSettingsBackendKeysChangedFunc          keys_changed,
+                          GSettingsBackendWritableChangedFunc      writable_changed,
+                          GSettingsBackendPathWritableChangedFunc  path_writable_changed)
+{
+  GSettingsBackendWatch *watch;
+
+  /* For purposes of discussion, we assume that our target is a
+   * GSettings instance.
+   *
+   * Our strategy to defend against the final reference dropping on the
+   * GSettings object in a thread other than the one that is doing the
+   * dispatching is as follows:
+   *
+   *  1) hold a GObject reference on the GSettings during an outstanding
+   *     dispatch.  This ensures that the delivery is always possible.
+   *
+   *  2) hold a weak reference on the GSettings at other times.  This
+   *     allows us to receive early notification of pending destruction
+   *     of the object.  At this point, it is still safe to obtain a
+   *     reference on the GObject to keep it alive, so #1 will work up
+   *     to that point.  After that point, we'll have been able to drop
+   *     the watch from the list.
+   *
+   * Note, in particular, that it's not possible to simply have an
+   * "unwatch" function that gets called from the finalize function of
+   * the GSettings instance because, by that point it is no longer
+   * possible to keep the object alive using g_object_ref() and we would
+   * have no way of knowing this.
+   *
+   * Note also that we do not need to hold a reference on the main
+   * context here since the GSettings instance does that for us and we
+   * will receive the weak notify long before it is dropped.  We don't
+   * even need to hold it during dispatches because our reference on the
+   * GSettings will prevent the finalize from running and dropping the
+   * ref on the context.
+   *
+   * All access to the list holds a mutex.  We have some strategies to
+   * avoid some of the pain that would be associated with that.
+   */
+
+  watch = g_slice_new (GSettingsBackendWatch);
+  watch->context = context;
+  watch->target = target;
+  g_object_weak_ref (target, g_settings_backend_watch_weak_notify, backend);
+
+  watch->changed = changed;
+  watch->path_changed = path_changed;
+  watch->keys_changed = keys_changed;
+  watch->writable_changed = writable_changed;
+  watch->path_writable_changed = path_writable_changed;
+
+  /* linked list prepend */
+  g_static_mutex_lock (&backend->priv->lock);
+  watch->next = backend->priv->watches;
+  backend->priv->watches = watch;
+  g_static_mutex_unlock (&backend->priv->lock);
+}
+
+void
+g_settings_backend_unwatch (GSettingsBackend *backend,
+                            GObject          *target)
+{
+  /* Our caller surely owns a reference on 'target', so the order of
+   * these two calls is unimportant.
+   */
+  g_object_weak_unref (target, g_settings_backend_watch_weak_notify, backend);
+  g_settings_backend_watch_weak_notify (backend, target);
+}
+
+static gboolean
+g_settings_backend_invoke_closure (gpointer user_data)
+{
+  GSettingsBackendClosure *closure = user_data;
+
+  closure->function (closure->backend, closure->target, closure->name,
+                     closure->data1, closure->data2);
+
+  closure->data1_free (closure->data1);
+  g_object_unref (closure->backend);
+  g_object_unref (closure->target);
+  g_free (closure->name);
+
+  g_slice_free (GSettingsBackendClosure, closure);
+
+  return FALSE;
+}
+
+static gpointer
+pointer_id (gpointer a)
+{
+  return a;
+}
+
+static void
+pointer_ignore (gpointer a)
+{
+}
+
+static void
+g_settings_backend_dispatch_signal (GSettingsBackend *backend,
+                                    gsize             function_offset,
+                                    const gchar      *name,
+                                    gpointer          data1,
+                                    GBoxedCopyFunc    data1_copy,
+                                    GBoxedFreeFunc    data1_free,
+                                    gpointer          data2)
+{
+  GMainContext *context, *here_and_now;
+  GSettingsBackendWatch *watch;
+
+  /* We need to hold the mutex here (to prevent a node from being
+   * deleted as we are traversing the list).  Since we should not
+   * re-enter user code while holding this mutex, we create a
+   * one-time-use GMainContext and populate it with the events that we
+   * would have called directly.  We dispatch these events after
+   * releasing the lock.  Note that the GObject reference is acquired on
+   * the target while holding the mutex and the mutex needs to be held
+   * as part of the destruction of any GSettings instance (via the weak
+   * reference handling).  This is the key to the safety of the whole
+   * setup.
+   */
+
+  if (data1_copy == NULL)
+    data1_copy = pointer_id;
+
+  if (data1_free == NULL)
+    data1_free = pointer_ignore;
+
+  context = g_settings_backend_get_active_context ();
+  here_and_now = g_main_context_new ();
+
+  /* traverse the (immutable while holding lock) list */
+  g_static_mutex_lock (&backend->priv->lock);
+  for (watch = backend->priv->watches; watch; watch = watch->next)
+    {
+      GSettingsBackendClosure *closure;
+      GSource *source;
+
+      closure = g_slice_new (GSettingsBackendClosure);
+      closure->backend = g_object_ref (backend);
+      closure->target = g_object_ref (watch->target);
+      closure->function = G_STRUCT_MEMBER (void *, watch, function_offset);
+      closure->name = g_strdup (name);
+      closure->data1 = data1_copy (data1);
+      closure->data1_free = data1_free;
+      closure->data2 = data2;
+
+      source = g_idle_source_new ();
+      g_source_set_priority (source, G_PRIORITY_DEFAULT);
+      g_source_set_callback (source,
+                             g_settings_backend_invoke_closure,
+                             closure, NULL);
+
+      if (watch->context && watch->context != context)
+        g_source_attach (source, watch->context);
+      else
+        g_source_attach (source, here_and_now);
+
+      g_source_unref (source);
+    }
+  g_static_mutex_unlock (&backend->priv->lock);
+
+  while (g_main_context_iteration (here_and_now, FALSE));
+  g_main_context_unref (here_and_now);
+}
+
 /**
  * g_settings_backend_changed:
  * @backend: a #GSettingsBackend implementation
@@ -215,13 +425,13 @@ g_settings_backend_changed (GSettingsBackend *backend,
                             const gchar      *key,
                             gpointer          origin_tag)
 {
-  GSettingsBackendWatch *watch;
-
   g_return_if_fail (G_IS_SETTINGS_BACKEND (backend));
   g_return_if_fail (is_key (key));
 
-  for (watch = backend->priv->watches; watch; watch = watch->next)
-    watch->changed (backend, key, origin_tag, watch->user_data);
+  g_settings_backend_dispatch_signal (backend,
+                                      G_STRUCT_OFFSET (GSettingsBackendWatch,
+                                                       changed),
+                                      key, origin_tag, NULL, NULL, NULL);
 }
 
 /**
@@ -261,14 +471,19 @@ g_settings_backend_keys_changed (GSettingsBackend    *backend,
                                  gchar const * const *items,
                                  gpointer             origin_tag)
 {
-  GSettingsBackendWatch *watch;
-
   g_return_if_fail (G_IS_SETTINGS_BACKEND (backend));
-  g_return_if_fail (path[0] == '\0' || is_path (path));
+  g_return_if_fail (is_path (path));
+
+  /* XXX: should do stricter checking (ie: inspect each item) */
   g_return_if_fail (items != NULL);
 
-  for (watch = backend->priv->watches; watch; watch = watch->next)
-    watch->keys_changed (backend, path, items, origin_tag, watch->user_data);
+  g_settings_backend_dispatch_signal (backend,
+                                      G_STRUCT_OFFSET (GSettingsBackendWatch,
+                                                       keys_changed),
+                                      path, (gpointer) items,
+                                      (GBoxedCopyFunc) g_strdupv,
+                                      (GBoxedFreeFunc) g_strfreev,
+                                      origin_tag);
 }
 
 /**
@@ -306,13 +521,13 @@ g_settings_backend_path_changed (GSettingsBackend *backend,
                                  const gchar      *path,
                                  gpointer          origin_tag)
 {
-  GSettingsBackendWatch *watch;
-
   g_return_if_fail (G_IS_SETTINGS_BACKEND (backend));
   g_return_if_fail (is_path (path));
 
-  for (watch = backend->priv->watches; watch; watch = watch->next)
-    watch->path_changed (backend, path, origin_tag, watch->user_data);
+  g_settings_backend_dispatch_signal (backend,
+                                      G_STRUCT_OFFSET (GSettingsBackendWatch,
+                                                       path_changed),
+                                      path, origin_tag, NULL, NULL, NULL);
 }
 
 /**
@@ -331,13 +546,13 @@ void
 g_settings_backend_writable_changed (GSettingsBackend *backend,
                                      const gchar      *key)
 {
-  GSettingsBackendWatch *watch;
-
   g_return_if_fail (G_IS_SETTINGS_BACKEND (backend));
   g_return_if_fail (is_key (key));
 
-  for (watch = backend->priv->watches; watch; watch = watch->next)
-    watch->writable_changed (backend, key, watch->user_data);
+  g_settings_backend_dispatch_signal (backend,
+                                      G_STRUCT_OFFSET (GSettingsBackendWatch,
+                                                       writable_changed),
+                                      key, NULL, NULL, NULL, NULL);
 }
 
 /**
@@ -357,28 +572,29 @@ void
 g_settings_backend_path_writable_changed (GSettingsBackend *backend,
                                           const gchar      *path)
 {
-  GSettingsBackendWatch *watch;
-
   g_return_if_fail (G_IS_SETTINGS_BACKEND (backend));
   g_return_if_fail (is_path (path));
 
-  for (watch = backend->priv->watches; watch; watch = watch->next)
-    watch->path_writable_changed (backend, path, watch->user_data);
+  g_settings_backend_dispatch_signal (backend,
+                                      G_STRUCT_OFFSET (GSettingsBackendWatch,
+                                                       path_writable_changed),
+                                      path, NULL, NULL, NULL, NULL);
 }
 
 typedef struct
 {
+  const gchar **keys;
+  GVariant **values;
   gint prefix_len;
   gchar *prefix;
-  gchar **items;
-} GetKeysState;
+} FlattenState;
 
 static gboolean
-tree_get_keys (gpointer key,
-               gpointer value,
-               gpointer user_data)
+g_settings_backend_flatten_one (gpointer key,
+                                gpointer value,
+                                gpointer user_data)
 {
-  GetKeysState *state = user_data;
+  FlattenState *state = user_data;
   const gchar *skey = key;
   gint i;
 
@@ -419,11 +635,59 @@ tree_get_keys (gpointer key,
   /* save the entire item into the array.
    * the prefixes will be removed later.
    */
-  *state->items++ = key;
+  *state->keys++ = key;
+
+  if (state->values)
+    *state->values++ = value;
 
   return FALSE;
 }
 
+/**
+ * g_settings_backend_flatten_tree:
+ * @tree: a #GTree containing the changes
+ * @path: the location to save the path
+ * @keys: the location to save the relative keys
+ * @values: the location to save the values, or %NULL
+ *
+ * Calculate the longest common prefix of all keys in a tree and write
+ * out an array of the key names relative to that prefix and,
+ * optionally, the value to store at each of those keys.
+ *
+ * You must free the value returned in @path, @keys and @values using
+ * g_free().  You should not attempt to free or unref the contents of
+ * @keys or @values.
+ *
+ * Since: 2.26
+ **/
+void
+g_settings_backend_flatten_tree (GTree         *tree,
+                                 gchar        **path,
+                                 const gchar ***keys,
+                                 GVariant    ***values)
+{
+  FlattenState state = { 0, };
+  gsize nnodes;
+
+  nnodes = g_tree_nnodes (tree);
+
+  *keys = state.keys = g_new (const gchar *, nnodes + 1);
+  state.keys[nnodes] = NULL;
+
+  if (values != NULL)
+    {
+      *values = state.values = g_new (GVariant *, nnodes + 1);
+      state.values[nnodes] = NULL;
+    }
+
+  g_tree_foreach (tree, g_settings_backend_flatten_one, &state);
+  g_return_if_fail (*keys + nnodes == state.keys);
+
+  *path = state.prefix;
+  while (nnodes--)
+    *--state.keys += state.prefix_len;
+}
+
 /**
  * g_settings_backend_changed_tree:
  * @backend: a #GSettingsBackend implementation
@@ -442,33 +706,38 @@ g_settings_backend_changed_tree (GSettingsBackend *backend,
                                  gpointer          origin_tag)
 {
   GSettingsBackendWatch *watch;
-  GetKeysState state = { 0, };
-  gchar **list;
+  const gchar **keys;
+  gchar *path;
 
-  list = g_new (gchar *, g_tree_nnodes (tree) + 1);
-  state.items = list;
+  g_return_if_fail (G_IS_SETTINGS_BACKEND (backend));
 
-  g_tree_foreach (tree, tree_get_keys, &state);
-  g_return_if_fail (list + g_tree_nnodes (tree) == state.items);
-  *state.items = NULL;
+  g_settings_backend_flatten_tree (tree, &path, &keys, NULL);
 
-  while (state.items-- != list)
-    *state.items += state.prefix_len;
+#ifdef DEBUG_CHANGES
+  {
+    gint i;
+
+    g_print ("----\n");
+    g_print ("changed_tree(): prefix %s\n", path);
+    for (i = 0; keys[i]; i++)
+      g_print ("  %s\n", keys[i]);
+    g_print ("----\n");
+  }
+#endif
 
   for (watch = backend->priv->watches; watch; watch = watch->next)
-    watch->keys_changed (backend, state.prefix,
-                         (const gchar * const *) list,
-                         origin_tag, watch->user_data);
+    watch->keys_changed (backend, watch->target, path, keys, origin_tag);
 
-  g_free (list);
-  g_free (state.prefix);
+  g_free (path);
+  g_free (keys);
 }
 
 /*< private >
  * g_settings_backend_read:
  * @backend: a #GSettingsBackend implementation
  * @key: the key to read
- * @expected_type: a #GVariantType hint
+ * @expected_type: a #GVariantType
+ * @default_value: if the default value should be returned
  * @returns: the value that was read, or %NULL
  *
  * Reads a key. This call will never block.
@@ -476,19 +745,32 @@ g_settings_backend_changed_tree (GSettingsBackend *backend,
  * If the key exists, the value associated with it will be returned.
  * If the key does not exist, %NULL will be returned.
  *
- * If @expected_type is given, it serves as a type hint to the backend.
- * If you expect a key of a certain type then you should give
- * @expected_type to increase your chances of getting it.  Some backends
- * may ignore this argument and return values of a different type; it is
- * mostly used by backends that don't store strong type information.
+ * The returned value will be of the type given in @expected_type.  If
+ * the backend stored a value of a different type then %NULL will be
+ * returned.
+ *
+ * If @default_value is %TRUE then this gets the default value from the
+ * backend (ie: the one that the backend would contain if
+ * g_settings_reset() were called).
  */
 GVariant *
 g_settings_backend_read (GSettingsBackend   *backend,
                          const gchar        *key,
-                         const GVariantType *expected_type)
+                         const GVariantType *expected_type,
+                         gboolean            default_value)
 {
-  return G_SETTINGS_BACKEND_GET_CLASS (backend)
-    ->read (backend, key, expected_type);
+  GVariant *value;
+
+  value = G_SETTINGS_BACKEND_GET_CLASS (backend)
+    ->read (backend, key, expected_type, default_value);
+
+  if G_UNLIKELY (value && !g_variant_is_of_type (value, expected_type))
+    {
+      g_variant_unref (value);
+      value = NULL;
+    }
+
+  return value;
 }
 
 /*< private >
@@ -642,54 +924,15 @@ g_settings_backend_subscribe (GSettingsBackend *backend,
     ->subscribe (backend, name);
 }
 
-static void
-g_settings_backend_set_property (GObject         *object,
-                                 guint            prop_id,
-                                 const GValue    *value,
-                                 GParamSpec      *pspec)
-{
-  GSettingsBackend *backend = G_SETTINGS_BACKEND (object);
-
-  switch (prop_id)
-    {
-    case PROP_CONTEXT:
-      backend->priv->context = g_value_dup_string (value);
-      break;
-
-    default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-      break;
-    }
-}
-
-static void
-g_settings_backend_get_property (GObject    *object,
-                                 guint       prop_id,
-                                 GValue     *value,
-                                 GParamSpec *pspec)
-{
-  GSettingsBackend *backend = G_SETTINGS_BACKEND (object);
-
-  switch (prop_id)
-    {
-    case PROP_CONTEXT:
-      g_value_set_string (value, backend->priv->context);
-      break;
-
-    default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-      break;
-    }
-}
-
 static void
 g_settings_backend_finalize (GObject *object)
 {
   GSettingsBackend *backend = G_SETTINGS_BACKEND (object);
 
-  g_free (backend->priv->context);
+  g_static_mutex_unlock (&backend->priv->lock);
 
-  G_OBJECT_CLASS (g_settings_backend_parent_class)->finalize (object);
+  G_OBJECT_CLASS (g_settings_backend_parent_class)
+    ->finalize (object);
 }
 
 static void
@@ -704,6 +947,7 @@ g_settings_backend_init (GSettingsBackend *backend)
   backend->priv = G_TYPE_INSTANCE_GET_PRIVATE (backend,
                                                G_TYPE_SETTINGS_BACKEND,
                                                GSettingsBackendPrivate);
+  g_static_mutex_init (&backend->priv->lock);
 }
 
 static void
@@ -714,32 +958,9 @@ g_settings_backend_class_init (GSettingsBackendClass *class)
   class->subscribe = ignore_subscription;
   class->unsubscribe = ignore_subscription;
 
-  gobject_class->get_property = g_settings_backend_get_property;
-  gobject_class->set_property = g_settings_backend_set_property;
   gobject_class->finalize = g_settings_backend_finalize;
 
   g_type_class_add_private (class, sizeof (GSettingsBackendPrivate));
-
-  /**
-   * GSettingsBackend:context:
-   *
-   * The "context" property gives a hint to the backend as to
-   * what storage to use. It is up to the implementation to make
-   * use of this information.
-   *
-   * E.g. DConf supports "user", "system", "defaults" and "login"
-   * contexts.
-   *
-   * If your backend supports different contexts, you should also
-   * provide an implementation of the supports_context() class
-   * function in #GSettingsBackendClass.
-   */
-  g_object_class_install_property (gobject_class, PROP_CONTEXT,
-    g_param_spec_string ("context", P_("Context"),
-                         P_("An identifier to decide which storage to use"),
-                         "", G_PARAM_READWRITE |
-                         G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_STRINGS));
-
 }
 
 /*< private >
@@ -757,174 +978,105 @@ g_settings_backend_create_tree (void)
                           g_free, (GDestroyNotify) g_variant_unref);
 }
 
-
-static gpointer
-get_default_backend (const gchar *context)
-{
-  GIOExtension *extension = NULL;
-  GIOExtensionPoint *point;
-  GList *extensions;
-  const gchar *env;
-  GType type;
-
-  _g_io_modules_ensure_loaded ();
-
-  point = g_io_extension_point_lookup (G_SETTINGS_BACKEND_EXTENSION_POINT_NAME);
-
-  if ((env = getenv ("GSETTINGS_BACKEND")))
-    {
-      extension = g_io_extension_point_get_extension_by_name (point, env);
-
-      if (extension == NULL)
-        g_warning ("Can't find GSettings backend '%s' given in "
-                   "GSETTINGS_BACKEND environment variable", env);
-    }
-
-  if (extension == NULL)
-    {
-      extensions = g_io_extension_point_get_extensions (point);
-
-      if (extensions == NULL)
-        g_error ("No GSettingsBackend implementations exist.");
-
-      extension = extensions->data;
-    }
-
-  if (context[0] != '\0') /* (context != "") */
-    {
-      GSettingsBackendClass *backend_class;
-      GTypeClass *class;
-
-      class = g_io_extension_ref_class (extension);
-      backend_class = G_SETTINGS_BACKEND_CLASS (class);
-
-      if (backend_class->supports_context == NULL ||
-          !backend_class->supports_context (context))
-        {
-          g_type_class_unref (class);
-          return NULL;
-        }
-
-      g_type_class_unref (class);
-    }
-
-  type = g_io_extension_get_type (extension);
-
-  return g_object_new (type, "context", context, NULL);
-}
-
-static GHashTable *g_settings_backends;
-
 /*< private >
- * g_settings_backend_get_with_context:
- * @context: a context that might be used by the backend to determine
- *     which storage to use, or %NULL to use the default storage
+ * g_settings_backend_get_default:
  * @returns: the default #GSettingsBackend
  *
  * Returns the default #GSettingsBackend. It is possible to override
  * the default by setting the <envar>GSETTINGS_BACKEND</envar>
  * environment variable to the name of a settings backend.
  *
- * The @context parameter can be used to indicate that a different
- * than the default storage is desired. E.g. the DConf backend lets
- * you use "user", "system", "defaults" and "login" as contexts.
- *
- * If @context is not supported by the implementation, this function
- * returns an instance of the #GSettingsMemoryBackend.
- * See g_settings_backend_supports_context(),
- *
- * The user does not own the return value and it must not be freed.
+ * The user gets a reference to the backend.
  */
 GSettingsBackend *
-g_settings_backend_get_with_context (const gchar *context)
+g_settings_backend_get_default (void)
 {
-  GSettingsBackend *backend;
+  static gsize backend;
 
-  g_return_val_if_fail (context != NULL, NULL);
-
-  _g_io_modules_ensure_extension_points_registered ();
-
-  if (g_settings_backends == NULL)
-    g_settings_backends = g_hash_table_new (g_str_hash, g_str_equal);
-
-  backend = g_hash_table_lookup (g_settings_backends, context);
-
-  if (!backend)
+  if (g_once_init_enter (&backend))
     {
-      backend = get_default_backend (context);
+      GSettingsBackend *instance;
+      GIOExtensionPoint *point;
+      GIOExtension *extension;
+      GType extension_type;
+      GList *extensions;
+      const gchar *env;
 
-      if (!backend)
-        backend = g_null_settings_backend_new ();
+      _g_io_modules_ensure_loaded ();
 
-      g_hash_table_insert (g_settings_backends, g_strdup (context), backend);
+      point = g_io_extension_point_lookup (G_SETTINGS_BACKEND_EXTENSION_POINT_NAME);
+      extension = NULL;
+
+      if ((env = getenv ("GSETTINGS_BACKEND")))
+        {
+          extension = g_io_extension_point_get_extension_by_name (point, env);
+
+          if (extension == NULL)
+            g_warning ("Can't find GSettings backend '%s' given in "
+                       "GSETTINGS_BACKEND environment variable", env);
+        }
+
+      if (extension == NULL)
+        {
+          extensions = g_io_extension_point_get_extensions (point);
+
+          if (extensions == NULL)
+            g_error ("No GSettingsBackend implementations exist.");
+
+          extension = extensions->data;
+
+          if (strcmp (g_io_extension_get_name (extension), "memory") == 0)
+            g_message ("Using the 'memory' GSettings backend.  Your settings "
+                       "will not be saved or shared with other applications.");
+        }
+
+      extension_type = g_io_extension_get_type (extension);
+      instance = g_object_new (extension_type, NULL);
+
+      g_once_init_leave (&backend, (gsize) instance);
     }
 
-  return g_object_ref (backend);
+  return g_object_ref ((void *) backend);
 }
 
 /*< private >
- * g_settings_backend_supports_context:
- * @context: a context string that might be passed to
- *     g_settings_backend_new_with_context()
- * @returns: #TRUE if @context is supported
+ * g_settings_backend_get_permission:
+ * @backend: a #GSettingsBackend
+ * @path: a path
+ * @returns: a non-%NULL #GPermission. Free with g_object_unref()
  *
- * Determines if the given context is supported by the default
- * GSettingsBackend implementation.
+ * Gets the permission object associated with writing to keys below
+ * @path on @backend.
+ *
+ * If this is not implemented in the backend, then a %TRUE
+ * #GSimplePermission is returned.
  */
-gboolean
-g_settings_backend_supports_context (const gchar *context)
+GPermission *
+g_settings_backend_get_permission (GSettingsBackend *backend,
+                                   const gchar      *path)
 {
+  GSettingsBackendClass *class = G_SETTINGS_BACKEND_GET_CLASS (backend);
+
+  if (class->get_permission)
+    return class->get_permission (backend, path);
+
+  return g_simple_permission_new (TRUE);
+}
+
+/*< private >
+ * g_settings_backend_sync_default:
+ *
+ * Syncs the default backend.
+ */
+void
+g_settings_backend_sync_default (void)
+{
+  GSettingsBackendClass *class;
   GSettingsBackend *backend;
 
-  g_return_val_if_fail (context != NULL, FALSE);
+  backend = g_settings_backend_get_default ();
+  class = G_SETTINGS_BACKEND_GET_CLASS (backend);
 
-  backend = get_default_backend (context);
-
-  if (backend)
-    {
-      g_object_unref (backend);
-      return TRUE;
-    }
-
-  return FALSE;
+  if (class->sync)
+    class->sync (backend);
 }
-
-/**
- * g_settings_backend_setup:
- * @context: a context string (not %NULL or "")
- * @backend: a #GSettingsBackend
- *
- * Sets up @backend for use with #GSettings.
- *
- * If you create a #GSettings with its context property set to @context
- * then it will use the backend given to this function.  See
- * g_settings_new_with_context().
- *
- * The backend must be set up before any settings objects are created
- * for the named context.
- *
- * It is not possible to specify a backend for the default context.
- *
- * This function takes a reference on @backend and never releases it.
- *
- * Since: 2.26
- **/
-void
-g_settings_backend_setup (const gchar      *context,
-                          GSettingsBackend *backend)
-{
-  g_return_if_fail (context[0] != '\0');
-
-  if (g_settings_backends == NULL)
-    g_settings_backends = g_hash_table_new (g_str_hash, g_str_equal);
-
-  if (g_hash_table_lookup (g_settings_backends, context))
-    g_error ("A GSettingsBackend already exists for context '%s'", context);
-
-  g_hash_table_insert (g_settings_backends,
-                       g_strdup (context),
-                       g_object_ref (backend));
-}
-
-#define __G_SETTINGS_BACKEND_C__
-#include "gioaliasdef.c"
