@@ -28,7 +28,9 @@
 #error "You must define G_SETTINGS_ENABLE_BACKEND before including <gio/gsettingsbackend.h>."
 #endif
 
-#include <glib-object.h>
+#define __GIO_GIO_H_INSIDE__
+#include <gio/giotypes.h>
+#undef __GIO_GIO_H_INSIDE__
 
 G_BEGIN_DECLS
 
@@ -58,46 +60,45 @@ G_BEGIN_DECLS
  **/
 typedef struct _GSettingsBackendPrivate                     GSettingsBackendPrivate;
 typedef struct _GSettingsBackendClass                       GSettingsBackendClass;
-typedef struct _GSettingsBackend                            GSettingsBackend;
-
 
 struct _GSettingsBackendClass
 {
   GObjectClass parent_class;
 
-  gboolean    (*supports_context) (const gchar         *context);
+  GVariant *    (*read)             (GSettingsBackend    *backend,
+                                     const gchar         *key,
+                                     const GVariantType  *expected_type,
+                                     gboolean             default_value);
+  gchar **      (*list)             (GSettingsBackend    *backend,
+                                     const gchar         *path,
+                                     gchar              **resets,
+                                     gsize                n_resets,
+                                     gsize               *length);
+  gboolean      (*write)            (GSettingsBackend    *backend,
+                                     const gchar         *key,
+                                     GVariant            *value,
+                                     gpointer             origin_tag);
+  gboolean      (*write_keys)       (GSettingsBackend    *backend,
+                                     GTree               *tree,
+                                     gpointer             origin_tag);
+  void          (*reset)            (GSettingsBackend    *backend,
+                                     const gchar         *key,
+                                     gpointer             origin_tag);
+  void          (*reset_path)       (GSettingsBackend    *backend,
+                                     const gchar         *path,
+                                     gpointer             origin_tag);
+  gboolean      (*get_writable)     (GSettingsBackend    *backend,
+                                     const gchar         *key);
+  void          (*subscribe)        (GSettingsBackend    *backend,
+                                     const gchar         *name);
+  void          (*unsubscribe)      (GSettingsBackend    *backend,
+                                     const gchar         *name);
+  void          (*sync)             (GSettingsBackend    *backend);
 
-  GVariant *  (*read)             (GSettingsBackend    *backend,
-                                   const gchar         *key,
-                                   const GVariantType  *expected_type,
-                                   gboolean             default_value);
-  gchar **    (*list)             (GSettingsBackend    *backend,
-                                   const gchar         *path,
-                                   gchar              **resets,
-                                   gsize                n_resets,
-                                   gsize               *length);
-  gboolean    (*write)            (GSettingsBackend    *backend,
-                                   const gchar         *key,
-                                   GVariant            *value,
-                                   gpointer             origin_tag);
-  gboolean    (*write_keys)       (GSettingsBackend    *backend,
-                                   GTree               *tree,
-                                   gpointer             origin_tag);
-  void        (*reset)            (GSettingsBackend    *backend,
-                                   const gchar         *key,
-                                   gpointer             origin_tag);
-  void        (*reset_path)       (GSettingsBackend    *backend,
-                                   const gchar         *path,
-                                   gpointer             origin_tag);
-  gboolean    (*get_writable)     (GSettingsBackend    *backend,
-                                   const gchar         *key);
-  void        (*subscribe)        (GSettingsBackend    *backend,
-                                   const gchar         *name);
-  void        (*unsubscribe)      (GSettingsBackend    *backend,
-                                   const gchar         *name);
-  void        (*sync)             (GSettingsBackend    *backend);
+  GPermission * (*get_permission)   (GSettingsBackend    *backend,
+                                     const gchar         *path);
 
-  gpointer padding[8];
+  gpointer padding[7];
 };
 
 struct _GSettingsBackend
@@ -108,36 +109,35 @@ struct _GSettingsBackend
   GSettingsBackendPrivate *priv;
 };
 
-GType                           g_settings_backend_get_type             (void);
+GType                   g_settings_backend_get_type                     (void);
 
-void                            g_settings_backend_setup                (const gchar         *context,
-                                                                         GSettingsBackend    *backend);
-void                            g_settings_backend_setup_keyfile        (const gchar         *context,
-                                                                         const gchar         *filename);
-
-void                            g_settings_backend_changed              (GSettingsBackend    *backend,
+void                    g_settings_backend_changed                      (GSettingsBackend    *backend,
                                                                          const gchar         *key,
                                                                          gpointer             origin_tag);
-void                            g_settings_backend_path_changed         (GSettingsBackend    *backend,
+void                    g_settings_backend_path_changed                 (GSettingsBackend    *backend,
                                                                          const gchar         *path,
                                                                          gpointer             origin_tag);
-void                            g_settings_backend_flatten_tree         (GTree               *tree,
+void                    g_settings_backend_flatten_tree                 (GTree               *tree,
                                                                          gchar              **path,
                                                                          const gchar       ***keys,
                                                                          GVariant          ***values);
-void                            g_settings_backend_keys_changed         (GSettingsBackend    *backend,
+void                    g_settings_backend_keys_changed                 (GSettingsBackend    *backend,
                                                                          const gchar         *path,
                                                                          gchar const * const *items,
                                                                          gpointer             origin_tag);
 
-void                            g_settings_backend_path_writable_changed(GSettingsBackend    *backend,
+void                    g_settings_backend_path_writable_changed        (GSettingsBackend    *backend,
                                                                          const gchar         *path);
-void                            g_settings_backend_writable_changed     (GSettingsBackend    *backend,
+void                    g_settings_backend_writable_changed             (GSettingsBackend    *backend,
                                                                          const gchar         *key);
-void                            g_settings_backend_changed_tree         (GSettingsBackend    *backend,
+void                    g_settings_backend_changed_tree                 (GSettingsBackend    *backend,
                                                                          GTree               *tree,
                                                                          gpointer             origin_tag);
 
+GSettingsBackend *      g_keyfile_settings_backend_new                  (const gchar         *filename,
+                                                                         const gchar         *root_path,
+                                                                         const gchar         *root_group);
+
 G_END_DECLS
 
 #endif /* __G_SETTINGS_BACKEND_H__ */
