@@ -26,7 +26,8 @@
  */
 
 #include "config.h"
-#include "glib.h"
+
+#include "gsocket.h"
 
 #include <errno.h>
 #include <signal.h>
@@ -42,7 +43,6 @@
 #include <sys/uio.h>
 #endif
 
-#include "gsocket.h"
 #include "gcancellable.h"
 #include "gioenumtypes.h"
 #include "ginetaddress.h"
@@ -55,7 +55,6 @@
 #include "gsocketcontrolmessage.h"
 #include "glibintl.h"
 
-#include "gioalias.h"
 
 /**
  * SECTION:gsocket
@@ -212,20 +211,13 @@ socket_strerror (int err)
 #ifndef G_OS_WIN32
   return g_strerror (err);
 #else
-  static GStaticPrivate msg_private = G_STATIC_PRIVATE_INIT;
-  char *buf, *msg;
-
-  buf = g_static_private_get (&msg_private);
-  if (!buf)
-    {
-      buf = g_new (gchar, 128);
-      g_static_private_set (&msg_private, buf, g_free);
-    }
+  static GStaticPrivate last_msg = G_STATIC_PRIVATE_INIT;
+  char *msg;
 
   msg = g_win32_error_message (err);
-  strncpy (buf, msg, 128);
-  g_free (msg);
-  return buf;
+  g_static_private_set (&last_msg, msg, g_free);
+
+  return msg;
 #endif
 }
 
@@ -2409,6 +2401,7 @@ socket_source_new (GSocket      *socket,
   condition |= G_IO_HUP | G_IO_ERR;
 
   source = g_source_new (&socket_source_funcs, sizeof (GSocketSource));
+  g_source_set_name (source, "GSocket");
   socket_source = (GSocketSource *)source;
 
   socket_source->socket = g_object_ref (socket);
@@ -2496,6 +2489,14 @@ g_socket_create_source (GSocket      *socket,
  * against the currently-satisfied conditions on @socket. The result
  * is returned.
  *
+ * Note that on Windows, it is possible for an operation to return
+ * %G_IO_ERROR_WOULD_BLOCK even immediately after
+ * g_socket_condition_check() has claimed that the socket is ready for
+ * writing. Rather than calling g_socket_condition_check() and then
+ * writing to the socket if it succeeds, it is generally better to
+ * simply try writing to the socket right away, and try again later if
+ * the initial attempt returns %G_IO_ERROR_WOULD_BLOCK.
+ *
  * It is meaningless to specify %G_IO_ERR or %G_IO_HUP in condition;
  * these conditions will always be set in the output if they are true.
  *
@@ -2962,7 +2963,7 @@ g_socket_send_message (GSocket                *socket,
  * @address: a pointer to a #GSocketAddress pointer, or %NULL
  * @vectors: an array of #GInputVector structs
  * @num_vectors: the number of elements in @vectors, or -1
- * @messages: a pointer which will be filled with an array of
+ * @messages: a pointer which may be filled with an array of
  *     #GSocketControlMessages, or %NULL
  * @num_messages: a pointer which will be filled with the number of
  *    elements in @messages, or %NULL
@@ -2990,12 +2991,13 @@ g_socket_send_message (GSocket                *socket,
  * single '\0' byte for the purposes of transferring ancillary data.
  *
  * @messages, if non-%NULL, will be set to point to a newly-allocated
- * array of #GSocketControlMessage instances. These correspond to the
- * control messages received from the kernel, one
- * #GSocketControlMessage per message from the kernel. This array is
- * %NULL-terminated and must be freed by the caller using g_free(). If
- * @messages is %NULL, any control messages received will be
- * discarded.
+ * array of #GSocketControlMessage instances or %NULL if no such
+ * messages was received. These correspond to the control messages
+ * received from the kernel, one #GSocketControlMessage per message
+ * from the kernel. This array is %NULL-terminated and must be freed
+ * by the caller using g_free() after calling g_object_unref() on each
+ * element. If @messages is %NULL, any control messages received will
+ * be discarded.
  *
  * @num_messages, if non-%NULL, will be set to the number of control
  * messages received.
@@ -3165,8 +3167,7 @@ g_socket_receive_message (GSocket                 *socket,
 
     /* decode control messages */
     {
-      GSocketControlMessage **my_messages = NULL;
-      gint allocated = 0, index = 0;
+      GPtrArray *my_messages = NULL;
       const gchar *scm_pointer;
       struct cmsghdr *cmsg;
       gsize scm_size;
@@ -3187,35 +3188,39 @@ g_socket_receive_message (GSocket                 *socket,
 	       deserialization code, so just continue */
 	    continue;
 
-	  if (index == allocated)
+	  if (messages == NULL)
 	    {
-	      /* estimated 99% case: exactly 1 control message */
-	      allocated = MAX (allocated * 2, 1);
-	      my_messages = g_new (GSocketControlMessage *, (allocated + 1));
+	      /* we have to do it this way if the user ignores the
+	       * messages so that we will close any received fds.
+	       */
+	      g_object_unref (message);
+	    }
+	  else
+	    {
+	      if (my_messages == NULL)
+		my_messages = g_ptr_array_new ();
+	      g_ptr_array_add (my_messages, message);
 	    }
-
-	  my_messages[index++] = message;
 	}
 
       if (num_messages)
-	*num_messages = index;
+	*num_messages = my_messages != NULL ? my_messages->len : 0;
 
       if (messages)
 	{
-	  my_messages[index++] = NULL;
-	  *messages = my_messages;
+	  if (my_messages == NULL)
+	    {
+	      *messages = NULL;
+	    }
+	  else
+	    {
+	      g_ptr_array_add (my_messages, NULL);
+	      *messages = (GSocketControlMessage **) g_ptr_array_free (my_messages, FALSE);
+	    }
 	}
       else
 	{
-	  gint i;
-
-	  /* free all those messages we just constructed.
-	   * we have to do it this way if the user ignores the
-	   * messages so that we will close any received fds.
-	   */
-	  for (i = 0; i < index; i++)
-	    g_object_unref (my_messages[i]);
-	  g_free (my_messages);
+	  g_assert (my_messages == NULL);
 	}
     }
 
@@ -3305,10 +3310,12 @@ g_socket_receive_message (GSocket                 *socket,
     if (flags != NULL)
       *flags = win_flags;
 
+    if (messages != NULL)
+      *messages = NULL;
+    if (num_messages != NULL)
+      *num_messages = 0;
+
     return bytes_received;
   }
 #endif
 }
-
-#define __G_SOCKET_C__
-#include "gioaliasdef.c"
