@@ -21,9 +21,13 @@
  * Modified by the GLib Team and others 1997-2000.  See the AUTHORS
  * file for a list of people on the GLib Team.  See the ChangeLog
  * files for a list of changes.  These files are distributed with
- * GLib at ftp://ftp.gtk.org/pub/gtk/. 
+ * GLib at ftp://ftp.gtk.org/pub/gtk/.
  */
 
+#if defined(G_DISABLE_SINGLE_INCLUDES) && !defined (__GLIB_H_INSIDE__) && !defined (GLIB_COMPILATION)
+#error "Only <glib.h> can be included directly."
+#endif
+
 #ifndef __G_CONVERT_H__
 #define __G_CONVERT_H__
 
@@ -75,7 +79,7 @@ gchar* g_convert_with_fallback (const gchar  *str,
 				gssize        len,            
 				const gchar  *to_codeset,
 				const gchar  *from_codeset,
-				gchar        *fallback,
+				const gchar  *fallback,
 				gsize        *bytes_read,     
 				gsize        *bytes_written,  
 				GError      **error) G_GNUC_MALLOC;
