@@ -29,7 +29,6 @@
 
 #include <string.h>
 
-#include "galias.h"
 
 /**
  * SECTION: gvarianttype
@@ -959,6 +958,23 @@ g_variant_type_is_dict_entry (const GVariantType *type)
   return g_variant_type_peek_string (type)[0] == '{';
 }
 
+/**
+ * g_variant_type_is_variant:
+ * @type: a #GVariantType
+ * @returns: %TRUE if @type is the variant type
+ *
+ * Determines if the given @type is the variant type.
+ *
+ * Since 2.24
+ **/
+gboolean
+g_variant_type_is_variant (const GVariantType *type)
+{
+  g_return_val_if_fail (g_variant_type_check (type), FALSE);
+
+  return g_variant_type_peek_string (type)[0] == 'v';
+}
+
 /**
  * g_variant_type_hash:
  * @type: a #GVariantType
@@ -1472,6 +1488,3 @@ g_variant_type_checked_ (const gchar *type_string)
   g_return_val_if_fail (g_variant_type_string_is_valid (type_string), NULL);
   return (const GVariantType *) type_string;
 }
-
-#define __G_VARIANT_TYPE_C__
-#include "galiasdef.c"
